//------------------------------------------------------------------------
// IMAGES : Unix/FLTK Image defs
//------------------------------------------------------------------------
//
//  GL-Friendly Node Builder (C) 2000-2007 Andrew Apted
//
//  Based on 'BSP 2.3' by Colin Reed, Lee Killough and others.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//------------------------------------------------------------------------
//
//  These images are from DOOM II and are (C) id software.
//
//------------------------------------------------------------------------

#include "local.h"

const char * pldie_image_data[] = 
{
/* width height ncolors chars_per_pixel */
"35 43 52 1",
/* colors */
"  c None",
"- c #6cb75f",
"a c #64675b",
"b c #64ad57",
"c c #4b4430",
"d c #1c2f14",
"e c #579249",
"f c #9b897b",
"g c #a29885",
"h c #4e833f",
"i c #745f3a",
"j c #81cc78",
"k c #447434",
"l c #ba8a5f",
"m c #71706e",
"n c #787878",
"o c #9c744e",
"p c #87e87f",
"q c #687052",
"r c #7fdc78",
"s c #4e4e4e",
"t c #c79268",
"u c #3f3f3f",
"v c #3f4923",
"w c #343434",
"x c #c72424",
"y c #292929",
"z c #670000",
"Y c #817f66",
"A c #654c30",
"B c #a87f58",
"C c #8a6844",
"D c #6c5734",
"E c #3a642f",
"F c #2f5223",
"G c #b6a594",
"H c #b3835b",
"I c #d89f73",
"J c #293f1c",
"K c #5f9f4e",
"L c #58523e",
"M c #957049",
"N c #892121",
"O c #575f44",
"P c #d7b399",
"Q c #c08e64",
"R c #a87456",
"S c #878a70",
"T c #2f3a14",
"V c #7b6449",
"W c #74785b",
"X c #74c368",
/* pixels */
"                   GGGGG           ",
"                 GGPGGfaf          ",
"                GGPPPGGmaf         ",
"               gYfffgPGfsGg        ",
"              ffAYVVYAGxfPf        ",
"               YVAAAVVYYGffPx      ",
"               AuusszxAVxVGg       ",
"              mgfffQxxxsAVY        ",
"         -jrr-KmmnnmNxnbjjjX-K     ",
"       bjjjjI-rtIIKXjrrrXrpppr-K   ",
"      bbXj--tPPIQMKjrrrrpjprjX-be  ",
"     eKbbeelttHRMibjrrrrr-jjXXX-K  ",
"     EkhktIItRMCih-rrrrr-hFFEkhhkh ",
"    CMtIPIIHMiiihKXXXXXbkFciCoCCiE ",
"   MllIIIIHMVkheKXKhkEEFFdcCQIloBE ",
"   MSBQQHoCDhKXrXrXbKehkFdcotIIBB  ",
"   WSBRMVVDkeeNKrbRNe-KhEJdoQIIlB  ",
"   qWCViDDkK-XrpppPXXkzhkFJiBBQQB  ",
"   OqqDDzkeehkEEFFFEhebKkEFDCooBRS ",
"    OOO zhksNNNNNNNNNEhbkEFVfGGgfY ",
"         zzzRHRRNRNNNszEeEFOoBolBW ",
"         uzqYMRRRRRMszzzdEd oQIIlo ",
"        OqYSqRRRCicDYWOzzdd oBIIQo ",
"       qSSgqMRRRRRCAOqSWFcO  MQQHo ",
"      qSqggOAAAAAAAEqOYYkOOO MlBo  ",
"     qqOOOOqWWWWWWWkEEOOFOsssMtIV  ",
"     qhqOOFqYYYYYYWkhehksuswwHIto  ",
"     hhhkkkOOOqqOOWKhKaasyyskDVHt  ",
"    eheKeKbkOOOOOOe-amnmswwFECDHH  ",
"    eb-bKXKEFJJJJFEssmmssuhEHoMHR  ",
"   hKb-bXhhhEFJJsaswwysubKehiHQI   ",
"  hKb---eeeEFdsaswwwywuKbbehEii    ",
"  Kb---KKeEFsaswwwyyuuebbbbekEJ    ",
" hb-bb-KekEsaswwyyssueKbbbbbKhkF   ",
"ebKbbbb-bJFsywyyssuwkeKbbbbb-bkFF  ",
"ehbbbeebFksswyysuww FFK-bbbbK-beOW ",
"EEeeeEEEhamasuwwwFyw F-bKbbKehbKSYW",
"OqqqOEJammasuwwJFFyw  -KeeehhkhESYW",
"uOOOOOsmmsswwwddwww   eehqqqqOOOqWO",
"TvLLLsssswwww         EFOOOOOOOOqqO",
" TTTssswwwy             TvLLLvvTvO ",
"    uwwww                TTTTTTcA  ",
"    sww                            "
};

const char * skull_image_data[] = 
{
/* width height ncolors chars_per_pixel */
"44 47 51 1",
/* colors */
"  c None",
"z c #000000",
"Y c #d00000",
"b c #d37000",
"c c #8e3b0b",
"d c #756853",
"e c #a33f00",
"f c #957f6a",
"g c #745f3a",
"h c #a9947c",
"i c #524929",
"j c #c70000",
"k c #d0bdaa",
"l c #ca6800",
"m c #a60000",
"n c #cab7a3",
"o c #f7a354",
"p c #c0ad99",
"q c #7f6c57",
"r c #7f643f",
"s c #f5d88f",
"t c #5f4e2f",
"u c #eb0000",
"v c #f70000",
"w c #e08021",
"x c #bd0000",
"y c #3a2f1c",
"Z c #9c0000",
"X c #99836f",
"A c #780000",
"B c #8a6844",
"C c #6c5734",
"D c #f18e2a",
"E c #e20000",
"F c #493f23",
"G c #b30000",
"H c #ffba7b",
"I c #d97814",
"J c #920000",
"K c #766150",
"L c #ba5a00",
"M c #d90000",
"N c #9f8a78",
"P c #a37852",
"Q c #8e7864",
"R c #2f2314",
"S c #870000",
"T c #af8153",
"U c #966e44",
"V c #705b3f",
"W c #b59f8b",
/* pixels */
"                      Y                     ",
"                      Y                     ",
"                    YYj                     ",
"                   uYj                      ",
"                 MMYYY                      ",
"                 ZGYjI                      ",
"                jZmljwZ                     ",
"                GGZIlwIju                   ",
"                ZjZwlIwGYjj                 ",
"              EGbYblYuuYvYjj                ",
"             MEDZbwbubuuEMGju               ",
"            xEIGGxbvIDwuvYxGY               ",
"            jjDZxluMwHDwMubxI Z             ",
"             ZjxMwvwDsHDwvbMDjZ             ",
"           EGZYxEDvDossoDvlExwGGE           ",
"          MMGZIxxovDHssHDIwxxMjGM           ",
"         xMYZjDYxovwHHHHwwDxYYMZYM          ",
"         jMlxvIxhbwDkkkkDDHhIbExlM          ",
"         GGwYgYWpnkkkkkkkknpbIbEwGG         ",
"QX      YjZDvYhWppnkkkkkknppWhwEDZjY      XQ",
"KQXWhhNffXNNTYhWppnkkkkkknppWhwTNNXffNhhWXQK",
" KQXXNXfQQqqQfNWWppnnnnnnppWWNfQqqQQfXNXXQK ",
"  KqQQQfQQqqCQfNhWWppppppWWhNfQCqqQQfQQQqK  ",
"    qqqeLlLeeqqQfNhhWWWWhhNfQqqeeLlLeqqq    ",
"       KiiiJJKKqQfXXNNNNXXfQqKKJJiiiK       ",
"         xgmJtVVKKKqqqqqqKKKVVtxmglZZ       ",
"        ZZTbjyBCCCCggggggCCCCByjlIYZ        ",
"        ZYlZSyPVVrrggggggrrVrPyAxMxZ        ",
"         ZZSAyPrrBrrrrrrrrBrBPyAAZZ         ",
"          jAyiPPPUUUUUUUUUUPPPiyAj          ",
"         ZuSgPFFFCrrPPPPrrCFFFPgSuZ         ",
"         ZSmiUALcRRRRFFRRRRcLAUimjZ         ",
"          SYtrAcTsYRFTBFRYsTcArtiS          ",
"          ZYiXJZYuSBTFiCTSuYZJXiiZ          ",
"          ZZFfhTTTBPiziiCPTTThfFi           ",
"          ZJFBCtCtirFFBFritCtCBFi           ",
"          ZMZBCiiFtrBTTBrtFiiCBxu           ",
"           jMAFFiirrrUrUrriiFAGIm           ",
"           JuGJRitrTPTPTPTtiRGuYZ           ",
"            AjbwTBlMszzsMlBTwbZZ            ",
"             ASBCllzzzzzzlICByy             ",
"              ygSJszzzzzzsJSgy              ",
"              yyUJoJsJJsJoJUyy              ",
"               idgJmjJJjmJgdi               ",
"                irggrrrrggri                ",
"                 iCUPTTPUCi                 ",
"                  iiiiiiii                  "
};

