/*----------------------------------------------------------------------------
 *  glBSPX MacOS X RESOURCE FILE
 *----------------------------------------------------------------------------
 *
 *  GL-Friendly Node Builder (C) 2000-2007 Andrew Apted
 *
 *  Based on 'BSP 2.3' by Colin Reed, Lee Killough and others.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *------------------------------------------------------------------------
 */

data 'MBAR' (128) {
	$"0001 0080"
};

data 'MENU' (128, "Apple") {
	$"0080 0000 0000 0000 0000 FFFF FFFB 0114"
	$"0A41 626F 7574 2046 4C54 4B00 0000 0001"
	$"2D00 0000 0000"
};

data 'carb' (0) {
};

